/** \file
    \author Shin'ichiro Nakaoka
*/

#ifndef CNOID_BODY_CONSTRAINT_FORCE_SOLVER_H_INCLUDED
#define CNOID_BODY_CONSTRAINT_FORCE_SOLVER_H_INCLUDED

#include "exportdecl.h"

namespace cnoid
{
    class Link;
    class CFSImpl;
    class WorldBase;
	
    class CNOID_EXPORT ConstraintForceSolver
    {
        CFSImpl* impl;
		
      public:
        ConstraintForceSolver(WorldBase& world);
        ~ConstraintForceSolver();
		
        bool addCollisionCheckLinkPair
            (int bodyIndex1, Link* link1, int bodyIndex2, Link* link2, double muStatic, double muDynamic, double culling_thresh, double epsilon);
        void clearCollisionCheckLinkPairs();

        void setGaussSeidelErrorCriterion(double e);
        void setGaussSeidelMaxNumIterations(int n);
        void setContactDepthCorrection(double depth, double velocityRatio);
        void setNegativeVelocityRatioForPenetration(double ratio);
        void set2Dmode(bool on);
        void enableConstraintForceOutput(bool on);

        static double defaultGaussSeidelErrorCriterion();
        static int defaultGaussSeidelMaxNumIterations();
        static double defaultContactCorrectionDepth();
        static double defaultContactCorrectionVelocityRatio();

        void initialize(void);
        void solve();
        void clearExternalForces();
    };
};


#endif
